# Copyright (C) 2018-2024 by BDE ENS Paris-Saclay
# SPDX-License-Identifier: GPL-3.0-or-later

from django.db import models, transaction
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from member.models import Club
from polymorphic.models import PolymorphicModel

#################################################################
# TO DO
# - link allergen with one food (basic or transformed) with check
# - check on basic food
# - check on transformed food
#################################################################


class QRCode(models.Model):
    """
    An QRCode model
    """
    qr_code_number = models.PositiveIntegerField(
        verbose_name=_("QR-code number"),
    )

    food_container = models.ForeignKey(
        'Food',
        on_delete=models.PROTECT,
        related_name='QR_code',
        unique=True,
        verbose_name=_('food container'),
    )

    class Meta:
        verbose_name = _("QR-code")
        verbose_name_plural = _("QR-codes")

    def __str__(self):
        return _("QR-code number {qr_code_number}").format(qr_code_number=self.qr_code_number)


class Allergen(models.Model):
    """
    A list of allergen and alimentary restrictions
    """
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255,
    )

    class Meta:
        verbose_name = _('Allergen')
        verbose_name_plural = _('Allergens')

    def __str__(self):
        return self.name


class Food(PolymorphicModel):
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255,
    )

    owner = models.ForeignKey(
        Club,
        on_delete=models.PROTECT,
        related_name='+',
        verbose_name=_('owner'),
    )

    allergens = models.ManyToManyField(
        Allergen,
        blank=True,
        verbose_name=_('allergen'),
    )

    expiry_date = models.DateTimeField(
        verbose_name=_('expiry date'),
    )

    was_eaten = models.BooleanField(
        default=False,
        verbose_name=_('was eaten'),
    )

    def __str__(self):
        return self.name

    @transaction.atomic
    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        return super().save(force_insert, force_update, using, update_fields)

    class Meta:
        verbose_name = _('food')
        verbose_name = _('foods')


class BasicFood(Food):
    """
    Food which has been directly buy on supermarket
    """
    date_type = models.CharField(
        max_length=255,
        choices=(
            ("DLC", "DLC"),
            ("DDM", "DDM"),
        )
    )

    arrival_date = models.DateTimeField(
        verbose_name=_('arrival date'),
        default=timezone.now,
        blank=True,  # TEMPORARY
    )

    # label = models.ImageField(
    #     verbose_name=_('food label'),
    #     max_length=255,
    #     blank=False,
    #     null=False,
    #     upload_to='label/',
    # )

    class Meta:
        verbose_name = _('Basic food')
        verbose_name_plural = _('Basic foods')


class TransformedFood(Food):
    """
    Transformed food  are a mix between basic food and meal
    """
    creation_date = models.DateTimeField(
        verbose_name=_('creation date'),
    )

    ingredient = models.ManyToManyField(
        Food,
        blank=True,
        symmetrical=False,
        related_name='transformed_ingredient_inv',
        verbose_name=_('transformed ingredient'),
    )

    is_active = models.BooleanField(
        default=True,
        verbose_name=_('is active'),
    )

    class Meta:
        verbose_name = _('Transformed food')
        verbose_name_plural = _('Transformed foods')
